==> choices_autocomplete-1.0.9.tar.gz: choices_autocomplete/js/choices.es6.js <==
((Drupal, once, Choices) => {
  /**
   * Provides Choices.js autocomplete plugin.
   */
  class ChoicesAutocomplete {
    /**
     * @param {HTMLSelectElement} element
     *   The select element.
     * @param {Object} labels
     *   An object of default option labels.
     */
    constructor(element, labels) {
      this.element = element;
      this.config = this.element.dataset;
      this.url = this.config.autocompletePath;
      this.cache = {};
      this.value = null;

      // Instantiate Choices.js.
      const config = JSON.parse(element.dataset.choices);
      config.loadingText = Drupal.theme(
        "ajaxProgressThrobber",
        config.loadingText
      );
      this.choices = new Choices(element, {
        ...config,
        shouldSort: false,
        allowHTML: true,
        removeItemButton: true,
        classNames: {
          containerOuter: "choices choices--autocomplete",
          item: "choices__item choices__item--autocomplete",
        },
      });
      if (this.choices.getValue().length) {
        this.choices.containerOuter.element.classList.add("is-filled");
      }
      element.addEventListener("change", () => {
        this.clear.bind(this);
        if (this.choices.getValue().length) {
          this.choices.containerOuter.element.classList.add("is-filled");
        } else {
          this.choices.containerOuter.element.classList.remove("is-filled");
        }
      });

      // Set default choice labels.
      if (labels) {
        const { choices } = this.choices.config;
        Object.keys(labels).forEach((value) => {
          choices.forEach((choice) => {
            if (choice.value === value) {
              choice.label = labels[value];
              choice.selected = true;
            }
          });
        });
        this.choices.removeActiveItems();
        this.choices.setChoices(choices, "value", "label", true);
      }

      // Detect max selected items.
      element.addEventListener("showDropdown", () => {
        if (this.isMax()) {
          this.setNotice(this.choices.config.maxItemText);
        }
      });
      element.addEventListener("addItem", () => {
        if (this.isMax()) {
          // Hide dropdown if at max selections.
          this.choices.hideDropdown();
        }
      });

      // Non-allowed and maxlength input filtering.
      this.choices.input.element.addEventListener("keydown", (event) => {
        if (
          !event.metaKey &&
          ((this.config.maxlength &&
            event.key.length === 1 &&
            event.target.value.length >= this.config.maxlength) ||
            (this.config.pattern &&
              event.key.length === 1 &&
              !event.key.match(new RegExp(this.config.pattern, "g"))))
        ) {
          event.preventDefault();
        }
      });

      // Bind to the search event.
      let timeout = null;
      element.addEventListener("search", (event) => {
        if (this.value !== event.detail.value) {
          this.clear();
          this.setNotice(this.choices.config.loadingText);
        }
        if (timeout) {
          clearTimeout(timeout);
        }
        timeout = setTimeout(this.search.bind(this, event), 500);
      });
    }

    /**
     * Set dropdown notice message.
     *
     * @param {string} message
     *   The message.
     */
    setNotice(message) {
      this.choices.choiceList.clear();
      this.choices.choiceList.append(
        this.choices._getTemplate("notice", message)
      );
    }

    /**
     * Check for max selected items.
     *
     * @return {boolean}
     *   Returns TRUE if max reached. Otherwise, FALSE.
     */
    isMax() {
      return (
        this.choices.config.maxItemCount > 1 &&
        this.choices.getValue(true).length >= this.choices.config.maxItemCount
      );
    }

    /**
     * Add or remove loading state.
     *
     * @param {boolean} loading
     *   Boolean indicating to add or remove loading state.
     */
    setState(loading) {
      if (loading) {
        this.choices.containerOuter.addLoadingState();
      } else {
        this.choices.containerOuter.removeLoadingState();
      }
    }

    /**
     * Search on user input.
     *
     * @param {CustomEvent} event
     *   The custom event.
     */
    search(event) {
      this.value = event.detail.value;

      if (this.cache[event.detail.value]) {
        // Populate from static cache.
        this.populate([...this.cache[event.detail.value]], event.detail.value);
      } else {
        this.setState(true);
        fetch(`${this.url}?q="${encodeURIComponent(event.detail.value)}"`, {
          headers: { "Content-Type": "application/json; charset=utf-8" },
        })
          .then((response) => response.json())
          .then((response) => {
            // Store response in static cache.
            this.cache[event.detail.value] = response;
            if (this.value === event.detail.value) {
              this.populate([...response], event.detail.value);
            }
          })
          .catch(() => {
            alert(Drupal.t("An error occurred. Please try again."));
          });
      }
    }

    /**
     * Populate with entity autocomplete items.
     *
     * @param {Array} items
     *   An array of items to populate.
     * @param {String} input
     *   The search input.
     */
    populate(items, input) {
      this.setState(false);
      let values = this.choices.getValue(true);
      if (!Array.isArray(values)) {
        values = typeof values !== "undefined" ? [values] : [];
      }

      // Gather current selection IDs and labels.
      const ids = [];
      const labels = values.map((value) => {
        const matches = value.match(/\s\(\d+\)"?$/);
        if (matches) {
          ids.push(matches[0].replace(/\D/g, "").trim());
        }
        return value
          .replace(/\s\(\d+\)"?$/, "")
          .toLowerCase()
          .trim();
      });

      // Remove non-unique items based on current selections.
      items = items.filter((item) => {
        labels.push(
          item.value
            .replace(/\s\(\d+\)"?$/, "")
            .toLowerCase()
            .trim()
        );
        const matches = item.value.match(/\s\(\d+\)"?$/);
        if (matches) {
          return ids.indexOf(matches[0].replace(/\W/g, "").trim()) === -1;
        }
        return labels.indexOf(item.value.toLowerCase().trim()) === -1;
      });

      // Allow selection of unique auto-created items.
      if (
        this.config.autocreate &&
        (!this.config.minlength || input.length >= this.config.minlength) &&
        labels.indexOf(input.toLowerCase().trim()) === -1
      ) {
        items.unshift({
          value: input.trim(),
          label: input.trim(),
        });
      }
      this.choices.setChoices(items, "value", "label", true);
    }

    /**
     * Clear populated entity autocomplete items.
     */
    clear() {
      this.choices.clearChoices();
    }
  }

  /**
   * Attach Choices.js autocomplete.
   */
  Drupal.behaviors.choicesAutocomplete = {
    attach: function attach(context, settings) {
      once(
        "choices-autocomplete",
        "select.js-choices-autocomplete",
        context
      ).forEach((element) => {
        element.choicesAutocomplete = new ChoicesAutocomplete(
          element,
          settings.choices_autocomplete
        );
      });
    },
  };
})(Drupal, once, Choices);

==> choices_autocomplete-1.1.1.tar.gz: choices_autocomplete/js/choices.es6.js <==
((Drupal, once, Choices) => {
  /**
   * Provides Choices.js autocomplete plugin.
   */
  class ChoicesAutocomplete {
    /**
     * @param {HTMLSelectElement} element
     *   The select element.
     * @param {Object} labels
     *   An object of default option labels.
     */
    constructor(element, labels) {
      this.element = element;
      this.config = this.element.dataset;
      this.url = this.config.autocompletePath;
      this.cache = {};
      this.value = null;

      // Instantiate Choices.js.
      const config = JSON.parse(element.dataset.choices);
      config.loadingText = Drupal.theme(
        "ajaxProgressThrobber",
        config.loadingText
      );
      this.choices = new Choices(element, {
        ...config,
        shouldSort: false,
        allowHTML: true,
        removeItemButton: true,
        classNames: {
          containerOuter: "choices choices--autocomplete",
          item: "choices__item choices__item--autocomplete",
        },
      });
      if (this.choices.getValue()) {
        this.choices.containerOuter.element.classList.add("is-filled");
      }
      element.addEventListener("change", () => {
        this.clear.bind(this);
        if (this.choices.getValue()) {
          this.choices.containerOuter.element.classList.add("is-filled");
        } else {
          this.choices.containerOuter.element.classList.remove("is-filled");
        }
      });

      // Set default choice labels.
      if (labels) {
        const { choices } = this.choices.config;
        Object.keys(labels).forEach((value) => {
          choices.forEach((choice) => {
            if (choice.value === value) {
              choice.label = labels[value];
              choice.selected = true;
            }
          });
        });
        this.choices.removeActiveItems();
        this.choices.setChoices(choices, "value", "label", true);
      }

      // Detect max selected items.
      element.addEventListener("showDropdown", () => {
        if (this.isMax()) {
          this.setNotice(this.choices.config.maxItemText);
        }
      });
      element.addEventListener("addItem", () => {
        if (this.isMax()) {
          // Hide dropdown if at max selections.
          this.choices.hideDropdown();
        }
      });

      // Non-allowed and maxlength input filtering.
      this.choices.input.element.addEventListener("keydown", (event) => {
        if (
          !event.metaKey &&
          ((this.config.maxlength &&
            event.key.length === 1 &&
            event.target.value.length >= this.config.maxlength) ||
            (this.config.pattern &&
              event.key.length === 1 &&
              !event.key.match(new RegExp(this.config.pattern, "g"))))
        ) {
          event.preventDefault();
        }
      });

      // Bind to the search event.
      let timeout = null;
      element.addEventListener("search", (event) => {
        if (this.value !== event.detail.value) {
          this.clear();
          this.setNotice(this.choices.config.loadingText);
        }
        if (timeout) {
          clearTimeout(timeout);
        }
        timeout = setTimeout(this.search.bind(this, event), 500);
      });
    }

    /**
     * Set dropdown notice message.
     *
     * @param {string} message
     *   The message.
     */
    setNotice(message) {
      this.choices.choiceList.clear();
      this.choices.choiceList.append(
        this.choices._getTemplate("notice", message)
      );
    }

    /**
     * Check for max selected items.
     *
     * @return {boolean}
     *   Returns TRUE if max reached. Otherwise, FALSE.
     */
    isMax() {
      return (
        this.choices.config.maxItemCount > 1 &&
        this.choices.getValue(true).length >= this.choices.config.maxItemCount
      );
    }

    /**
     * Add or remove loading state.
     *
     * @param {boolean} loading
     *   Boolean indicating to add or remove loading state.
     */
    setState(loading) {
      if (loading) {
        this.choices.containerOuter.addLoadingState();
      } else {
        this.choices.containerOuter.removeLoadingState();
      }
    }

    /**
     * Search on user input.
     *
     * @param {CustomEvent} event
     *   The custom event.
     */
    search(event) {
      this.value = event.detail.value;

      if (this.cache[event.detail.value]) {
        // Populate from static cache.
        this.populate([...this.cache[event.detail.value]], event.detail.value);
      } else {
        this.setState(true);
        fetch(`${this.url}?q="${encodeURIComponent(event.detail.value)}"`, {
          headers: { "Content-Type": "application/json; charset=utf-8" },
        })
          .then((response) => response.json())
          .then((response) => {
            // Store response in static cache.
            this.cache[event.detail.value] = response;
            if (this.value === event.detail.value) {
              this.populate([...response], event.detail.value);
            }
          })
          .catch(() => {
            alert(Drupal.t("An error occurred. Please try again."));
          });
      }
    }

    /**
     * Populate with entity autocomplete items.
     *
     * @param {Array} items
     *   An array of items to populate.
     * @param {String} input
     *   The search input.
     */
    populate(items, input) {
      this.setState(false);
      let values = this.choices.getValue(true);
      if (!Array.isArray(values)) {
        values = typeof values !== "undefined" ? [values] : [];
      }

      // Gather current selection IDs and labels.
      const ids = [];
      const labels = values.map((value) => {
        const matches = value.match(/\s\(\d+\)"?$/);
        if (matches) {
          ids.push(matches[0].replace(/\D/g, "").trim());
        }
        return value
          .replace(/\s\(\d+\)"?$/, "")
          .toLowerCase()
          .trim();
      });

      // Remove non-unique items based on current selections.
      items = items.filter((item) => {
        labels.push(
          item.value
            .replace(/\s\(\d+\)"?$/, "")
            .toLowerCase()
            .trim()
        );
        const matches = item.value.match(/\s\(\d+\)"?$/);
        if (matches) {
          return ids.indexOf(matches[0].replace(/\W/g, "").trim()) === -1;
        }
        return labels.indexOf(item.value.toLowerCase().trim()) === -1;
      });

      // Allow selection of unique auto-created items.
      if (
        this.config.autocreate &&
        (!this.config.minlength || input.length >= this.config.minlength) &&
        labels.indexOf(input.toLowerCase().trim()) === -1
      ) {
        items.unshift({
          value: input.trim(),
          label: input.trim(),
        });
      }
      this.choices.setChoices(items, "value", "label", true);
    }

    /**
     * Clear populated entity autocomplete items.
     */
    clear() {
      this.choices.clearChoices();
    }
  }

  /**
   * Attach Choices.js autocomplete.
   */
  Drupal.behaviors.choicesAutocomplete = {
    attach: function attach(context, settings) {
      once(
        "choices-autocomplete",
        "select.js-choices-autocomplete",
        context
      ).forEach((element) => {
        element.choicesAutocomplete = new ChoicesAutocomplete(
          element,
          settings.choices_autocomplete
        );
      });
    },
  };
})(Drupal, once, Choices);
